==> kpogre-1.2.7.tar.gz: kpogre-1.2.7/kpogre/DbObjects/kpgtablesfolder.cpp <==
/***************************************************************************
                          kpgtablesfolder  -  description
                             -------------------
    begin                : Ne no 1 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kpgtablesfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgserver.h"
#include "kpgconnection.h"
#include "kpgschema.h"
#include "kpgtable.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGTablesFolder::KPGTablesFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Tables"))
{
		m_oidNameSpace = oidNameSpace;
}


KPGTablesFolder::~KPGTablesFolder()
{
}

bool KPGTablesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items (databases)
  while (QListViewItem * pItem = firstChild())
    delete pItem;
		
	// Get pointer to server for version info
  KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent());

	// obtain list of tables
  QString strQuery("SELECT rel.oid, relname, description, pg_get_userbyid(relowner) AS relowner, relacl, ");
	strQuery.append("relhasoids, relhassubclass, reltuples, conname, conkey ");
	
	if(pServer->versionMajor() >= 8)
    strQuery.append(", rel.reltablespace, ts.spcname ");
		
	strQuery.append("FROM pg_class rel ");
	strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=rel.oid AND des.objsubid=0 ");
	strQuery.append("LEFT OUTER JOIN pg_constraint c ON c.conrelid=rel.oid AND c.contype='p' ");
	
	if(pServer->versionMajor() >= 8)
    strQuery.append("LEFT JOIN pg_tablespace ts ON rel.reltablespace=ts.oid ");
		
	strQuery.append("WHERE ((relkind = 'r') OR (relkind = 's')) AND relnamespace = " + QString("%1").arg(m_oidNameSpace));
		
  try
  {
    m_pqxxResultTables = pqxxXaction.Exec(strQuery.latin1());

		KPGTable *pgTable = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultTables.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultTables[i][0].to(oid);
		
			if(pgTable == 0)
      	pgTable = new KPGTable(this, m_pqxxResultTables[i]["relname"].c_str(), oid);
			else
				pgTable = new KPGTable(this, pgTable, m_pqxxResultTables[i]["relname"].c_str(), oid);
			
			bool bHasOids;
			bool bHasSubclass;
			long lEstimatedRows;
			
			m_pqxxResultTables[i]["relhasoids"].to(bHasOids);
			m_pqxxResultTables[i]["relhassubclass"].to(bHasSubclass);
			m_pqxxResultTables[i]["reltuples"].to(lEstimatedRows);
			
			if(pServer->versionMajor() <= 7)
			{
				pgTable->setProperties(
					m_pqxxResultTables[i]["relowner"].c_str(), 
					m_pqxxResultTables[i]["relacl"].c_str(),
					m_pqxxResultTables[i]["description"].c_str(), 
					bHasOids,
					bHasSubclass,
					lEstimatedRows,
					m_pqxxResultTables[i]["conname"].c_str(),
					m_pqxxResultTables[i]["conkey"].c_str() 
					);
			}
			else
			{
				pqxx::oid oidTablespace;
				m_pqxxResultTables[i]["reltablespace"].to(oidTablespace);
				
				pgTable->setProperties(
					m_pqxxResultTables[i]["relowner"].c_str(), 
					m_pqxxResultTables[i]["relacl"].c_str(),
					m_pqxxResultTables[i]["description"].c_str(), 
					bHasOids,
					bHasSubclass,
					lEstimatedRows,
					m_pqxxResultTables[i]["conname"].c_str(),
					m_pqxxResultTables[i]["conkey"].c_str(),
					oidTablespace,
					m_pqxxResultTables[i]["spcname"].c_str()
					);
			}
			
			//kdDebug() << "------- START: " << m_pqxxResultTables[i]["relname"].c_str() << endl;	
			//kdDebug() << "columns " << endl;	
      pgTable->refreshColumns(pqxxXaction);
			//kdDebug() << "constraints " << endl;	
			pgTable->refreshConstraints(pqxxXaction);
			//kdDebug() << "indexes " << endl;	
			pgTable->refreshIndexes(pqxxXaction);
			//kdDebug() << "rules " << endl;	
			pgTable->refreshRules(pqxxXaction);
			//kdDebug() << "triggers " << endl;	
			pgTable->refreshTriggers(pqxxXaction);
			//kdDebug() << "------- END: " << endl;
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGTablesFolder::displayProperties(QWidget *widgetProperies)
{
	// Get pointer to server for version info
  KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent());

  // Display tables list
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  pPropertyPage->setLabelText(i18n("Tables &list:"));
	
	//----------------------------------------------------------------------------
	//Display tebles list
	QTable *pTableTables = pPropertyPage->m_pTableItems;
  	
	// fill properties table with result data
  int nTableCols = fillPropertiesTable(m_pqxxResultTables, pTableTables);

  
  // field names
  QHeader* hHeader = pTableTables->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
  hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Has OIDs"));
  hHeader->setLabel(nCol++, i18n("Has subclass"));
  hHeader->setLabel(nCol++, i18n("Estimated rows"));
  hHeader->setLabel(nCol++, i18n("Primary key name"));
  hHeader->setLabel(nCol++, i18n("Primary key columns"));
  if(pServer->versionMajor() >= 8) hHeader->setLabel(nCol++, i18n("Tablespace"));
  
  
  for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableTables->adjustColumn(nCol);
  }
}

==> kpogre-1.5.2.tar.gz: kpogre-1.5.2/kpogre/DbObjects/kpgtablesfolder.cpp <==
/***************************************************************************
                          kpgtablesfolder  -  description
                             -------------------
    begin                : Ne no 1 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kpgtablesfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgserver.h"
#include "kpgconnection.h"
#include "kpgschema.h"
#include "kpgtable.h"
#include "../PropertyPages/kpgpropertypage.h"
#include "../kpgsqldialog.h"

KPGTablesFolder::KPGTablesFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Tables"))
{
		m_oidNameSpace = oidNameSpace;
}


KPGTablesFolder::~KPGTablesFolder()
{
}

bool KPGTablesFolder::refresh(pqxx::work &pqxxXaction)
{
    // delete all child items (tables)
    while (QListViewItem * pItem = firstChild())
        delete pItem;
		
	// Get pointer to server for version info
    KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent());

	// obtain list of tables
    QString strQuery("SELECT rel.oid, relname, description, pg_get_userbyid(relowner) AS relowner, relacl, ");
	strQuery.append("relhasoids, relhassubclass, reltuples, conname, conkey ");
	
	if(pServer->versionMajor() >= 8)
    strQuery.append(", rel.reltablespace, ts.spcname ");
		
	strQuery.append("FROM pg_catalog.pg_class rel ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_description des ON des.objoid=rel.oid AND des.objsubid=0 ");
	strQuery.append("LEFT OUTER JOIN pg_catalog.pg_constraint c ON c.conrelid=rel.oid AND c.contype='p' ");
	
	if(pServer->versionMajor() >= 8)
    strQuery.append("LEFT JOIN pg_catalog.pg_tablespace ts ON rel.reltablespace=ts.oid ");
		
	strQuery.append("WHERE ((relkind = 'r') OR (relkind = 's')) AND relnamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY relname");
		
  	try
  	{
    	m_pqxxResultTables = pqxxXaction.Exec(strQuery.latin1());

		KPGTable *pgTable = 0;
		
    	for (result::size_type i = 0; i != m_pqxxResultTables.size(); ++i)
    	{
			pqxx::oid oid;
			m_pqxxResultTables[i][0].to(oid);
		
			if(pgTable == 0)
      		    pgTable = new KPGTable(this, m_pqxxResultTables[i]["relname"].c_str(), oid);
			else
				pgTable = new KPGTable(this, pgTable, m_pqxxResultTables[i]["relname"].c_str(), oid);
			
			bool bHasOids;
			bool bHasSubclass;
			double dEstimatedRows;
			
			m_pqxxResultTables[i]["relhasoids"].to(bHasOids);
			m_pqxxResultTables[i]["relhassubclass"].to(bHasSubclass);
			m_pqxxResultTables[i]["reltuples"].to(dEstimatedRows);
			
			if(pServer->versionMajor() <= 7)
			{
				pgTable->setProperties(
					m_pqxxResultTables[i]["relowner"].c_str(), 
					m_pqxxResultTables[i]["relacl"].c_str(),
					m_pqxxResultTables[i]["description"].c_str(), 
					bHasOids,
					bHasSubclass,
					dEstimatedRows,
					m_pqxxResultTables[i]["conname"].c_str(),
					m_pqxxResultTables[i]["conkey"].c_str() 
					);
			}
			else
			{
				pqxx::oid oidTablespace;
				m_pqxxResultTables[i]["reltablespace"].to(oidTablespace);
				
				pgTable->setProperties(
					m_pqxxResultTables[i]["relowner"].c_str(), 
					m_pqxxResultTables[i]["relacl"].c_str(),
					m_pqxxResultTables[i]["description"].c_str(), 
					bHasOids,
					bHasSubclass,
					dEstimatedRows,
					m_pqxxResultTables[i]["conname"].c_str(),
					m_pqxxResultTables[i]["conkey"].c_str(),
					oidTablespace,
					m_pqxxResultTables[i]["spcname"].c_str()
					);
			}
			
			//kdDebug() << "------- START: " << m_pqxxResultTables[i]["relname"].c_str() << endl;	
			//kdDebug() << "columns " << endl;	
      		pgTable->refreshColumns(pqxxXaction);
			//kdDebug() << "constraints " << endl;	
			pgTable->refreshConstraints(pqxxXaction);
			//kdDebug() << "indexes " << endl;	
			pgTable->refreshIndexes(pqxxXaction);
			//kdDebug() << "rules " << endl;	
			pgTable->refreshRules(pqxxXaction);
			//kdDebug() << "triggers " << endl;	
			pgTable->refreshTriggers(pqxxXaction);
			//kdDebug() << "------- END: " << endl;
    	}
  	}
  	catch (const std::exception &e)
  	{
		kdError() << e.what() << endl;
    	KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    	return false;
  	} 
  
	return true;
}

void KPGTablesFolder::displayProperties(QWidget *widgetProperies)
{
	if(widgetProperies == 0) return; // be defensive, user may close Properties window
	
	// Get pointer to server for version info
  	KPGServer *pServer = static_cast <KPGServer *> (parent()->parent()->parent());

  	// Display tables list
	KPGPropertyPage *pPropertyPage = static_cast <KPGPropertyPage *> (widgetProperies);
  	pPropertyPage->setLabelText(i18n("Tables &list:"));
	
	//----------------------------------------------------------------------------
	//Display tebles list
	QTable *pTableTables = pPropertyPage->m_pTableItems;
  	
	// fill properties table with result data
  	int nTableCols = fillPropertiesTable(m_pqxxResultTables, pTableTables);

  
  	// field names
  	QHeader* hHeader = pTableTables->horizontalHeader();

  	int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  	hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Owner"));
  	hHeader->setLabel(nCol++, i18n("ACL"));
	hHeader->setLabel(nCol++, i18n("Has OIDs"));
  	hHeader->setLabel(nCol++, i18n("Has subclass"));
  	hHeader->setLabel(nCol++, i18n("Estimated rows"));
  	hHeader->setLabel(nCol++, i18n("Primary key name"));
  	hHeader->setLabel(nCol++, i18n("Primary key columns"));
  	if(pServer->versionMajor() >= 8) hHeader->setLabel(nCol++, i18n("Tablespace"));
  
  
  	for(int nCol = 0; nCol < nTableCols; nCol++)
  	{
    	pTableTables->adjustColumn(nCol);
  	}
}
